@@ -1,12 +1,11 @@
-#!/usr/bin/python
+#!/usr/bin/env python
 #@+leo-ver=4
 #@+node:@file gmailfs.py
 #@@first
 #
 #    Copyright (C) 2004  Richard Jones  <richard followed by funny at sign then jones then a dot then name>
 #
-#    GmailFS - Gmail Filesystem Version 0.3
-#
+#    GmailFS - Gmail Filesystem Version 0.6
 #    This program can be distributed under the terms of the GNU GPL.
 #    See the file COPYING.
 #
@@ -22,11 +21,12 @@ from fuse import Fuse
 import os
 from errno import *
 from stat import *
+from os.path import abspath, expanduser
 
 import thread
 import quopri
 import libgmail
-from constants import *
+from lgconstants import *
 
 import sys,traceback,re,string,time,tempfile,array,logging,logging.handlers
    
@@ -37,21 +37,13 @@ except:
 
 #@-node:imports
 
-# Setup logging
-log = logging.getLogger('gmailfs')
-
-loghdlr = logging.handlers.RotatingFileHandler("/var/log/gmailfs", "a", 5242880, 3)
-logfmt = logging.Formatter("%(asctime)s %(levelname)-10s %(message)s", "%x %X")
-loghdlr.setFormatter(logfmt)
-log.addHandler(loghdlr)
-log.setLevel(logging.WARNING)
-
 # Globals
 DefaultUsername = 'defaultUser'
 DefaultPassword = 'defaultPassword'
 DefaultFsname = 'gmailfs'
+References={}
 
-GMAILFS_VERSION = '2'
+GMAILFS_VERSION = '3'
 
 PathStartDelim  = '__a__'
 PathEndDelim    = '__b__'
@@ -85,6 +77,12 @@ NumberQueryRetries = 3
 
 regexObjectTrailingMB = re.compile(r'\s?MB$')
 
+def _addLoggingHandlerHelper(handler):
+    """ Sets our default formatter on the log handler before adding it to
+        the log object. """
+    handler.setFormatter(defaultLogFormatter)
+    log.addHandler(handler)
+
 def GmailConfig(fname):
     import ConfigParser
     cp = ConfigParser.ConfigParser()
@@ -96,44 +94,48 @@ def GmailConfig(fname):
     except:
       log.warning("Unable to read configuration file: " + fname)
       return
-    try:
+
+    sections = cp.sections()
+    if "account" in sections:
       options = cp.options("account")
       if "username" in options:
           DefaultUsername = cp.get("account", "username")
       if "password" in options:
           DefaultPassword = cp.get("account", "password")
-    except:
-      log.error("Unable to find GMail account configuation")
-    try:
+    else:
+      log.error("Unable to find GMail account configuration")
+
+    if "filesystem" in sections:
       options = cp.options("filesystem")
       if "fsname" in options:
           DefaultFsname = cp.get("filesystem", "fsname")
-    except:
+    else:
       log.warning("Using default file system (Dangerous!)")
-    try:
+      
+    if "logs" in sections:
       options = cp.options("logs")
       if "level" in options:
-          level = cp.get("logs", "level")
-          log.setLevel(logging._levelNames[level])
-    except:
-      pass
-    try:
+        level = cp.get("logs", "level")
+        log.setLevel(logging._levelNames[level])
+      if "logfile" in options:
+        logfile = abspath(expanduser(cp.get("logs", "logfile")))
+        log.removeHandler(defaultLoggingHandler)
+        _addLoggingHandlerHelper(logging.handlers.RotatingFileHandler(logfile, "a", 5242880, 3))
+        
+    if "connection" in sections:
       options = cp.options("connection")
-      if "proxy" in options:
-          try:
-              OpenSSLProxy.OpenSSLInstallHandler(cp.get("connection", "proxy"))
-          except:
-              log.error("OpenSSLProxy is missing. Can't use HTTPS proxy!");
-      else:
-          try:
-              OpenSSLProxy.OpenSSLInstallHandler()
-          except:
-              log.warning("OpenSSLProxy is missing. HTTPS proxy support disabled.");
+      try:
+        if "proxy" in options:
+          OpenSSLProxy.OpenSSLInstallHandler(cp.get("connection", "proxy"))
+        else:
+          OpenSSLProxy.OpenSSLInstallHandler()
+      except:
+        log.error("OpenSSLProxy is missing. Can't use HTTPS proxy!");
+
       if "retries" in options:
-          NumberQueryRetries = string.atoi(cp.get("connection", "retries"))
-    except:
-      pass
-    try:
+          NumberQueryRetries = cp.getint("connection", "retries")
+
+    if "references" in sections:
       options = cp.options("references")
       for option in options:
           record = cp.get("references",option)
@@ -143,8 +145,6 @@ def GmailConfig(fname):
               continue
           reference = reference_class(*fields)
           References[option] = reference
-    except:
-      pass
                                                                                 
 class reference_class:
     def __init__(self,fsname,username=None,password=None):
@@ -158,14 +158,6 @@ class reference_class:
       else:
           self.password = password
 
-References={}
-GmailConfig("/etc/gmailfs.conf")
-try:
-    libgmail.ConfigLogs(log)
-except:
-    pass
-
-
 # This ensures backwards compatability where
 # old filesystems were stored with 7bit encodings
 # but new ones are all quoted printable
@@ -540,14 +532,19 @@ class Gmailfs(Fuse):
 
     
     #@+node:__init__
-    def __init__(self, *args, **kw):
+    def __init__(self, mountpoint, odata):
+
+        # FIXME -- Sebastien Delafond <seb@debian.org>
+        # dirty hack until I change the python-fuse constructor to also
+        # handle its own parameters, and not only the command-line ones. 
+        sys.argv[1] = mountpoint
+        sys.argv[2] = odata
     
-        Fuse.__init__(self, *args, **kw)
+        Fuse.__init__(self)
     
-        if 1:
-            print "gmailfs.py:Gmailfs:mountpoint: %s" % repr(self.mountpoint)
-            print "gmailfs.py:Gmailfs:unnamed mount options: %s" % self.optlist
-            print "gmailfs.py:Gmailfs:named mount options: %s" % self.optdict
+        log.info("Mountpoint: %s" % self.mountpoint)
+        log.info("Unnamed mount options: %s" % self.optlist)
+        log.info("Named mount options: %s" % self.optdict)
     
         # do stuff to set up your filesystem here, if you want
 
@@ -879,7 +876,7 @@ class Gmailfs(Fuse):
     def mknod(self, path, mode, dev):
     	""" Python has no os.mknod, so we can only do some things """
         log.debug("mknod:"+path)
-    	if S_ISREG(mode):
+	if S_ISREG(mode) | S_ISFIFO(mode) | S_ISSOCK(mode):
             self._mkfileOrDir(path,None,mode,-1,0,1)
     	    #open(path, "w")
     	else:
@@ -1113,11 +1110,31 @@ class Gmailfs(Fuse):
 #@-node:class Gmailfs
 #@+node:mainline
 
+# Setup logging
+log = logging.getLogger('gmailfs')
+defaultLogLevel = logging.WARNING
+log.setLevel(defaultLogLevel)
+defaultLogFormatter = logging.Formatter("%(asctime)s %(levelname)-10s %(message)s", "%x %X")
+    
+# log to stdout while parsing the config while
+defaultLoggingHandler = logging.StreamHandler(sys.stdout)
+_addLoggingHandlerHelper(defaultLoggingHandler)
+
+GmailConfig("/etc/gmailfs.conf")
+try:
+    libgmail.ConfigLogs(log)
+except:
+    pass
+
+def main(mountpoint, odata):
+    server = Gmailfs(mountpoint, odata)
+    server.flags = 0
+    server.multithreaded = 0;
+    server.main()
+
 if __name__ == '__main__':
-	server = Gmailfs()
-	server.flags = 0
-	server.multithreaded = 1;
-	server.main()
+    main()
+    
 #@-node:mainline
 #@-others
 #@-node:@file gmailfs.py
